`timescale 1ns/1ns
`define NULL 0
`define EOF 32'hffff_ffff

module test1;

   parameter    clock_period = 5;

   //Variables required for file handling
   integer      filein,mismatch,c,r,w,i;
   reg 		clock;
   reg 		rst;
   reg 		in_flag;
   reg [ 7:0] 	in_data;
   wire [ 7:0]	out_data;
   reg [ 7:0] 	out_data_expected;


   COUNT U0 ( .clock      (clock ),
              .rst        (rst        ),
              .in_flag    (in_flag    ),
              .in_data    (in_data    ),
              .out_data   (out_data   )
	      );

   initial
     begin
	clock = 1;
	mismatch = 0;	

	// Open a stimulus file, and save the file handle
	filein = $getpat_open("test1.hex", "x", rst,in_flag,in_data,out_data_expected);
	if (filein == `NULL)
	  begin
	     $display ("Can't open input file 'test1.hex'");
	     $finish;
	  end
     end

   always
     begin
	#clock_period clock = ~clock;
     end

   always @(negedge clock)
     begin

	// Read the next line from the file
	r = $getpat_next(filein);

	if ( r == `EOF )
	  begin
	     $display ("Reached end of file");
	     $display("Found %0d mismatches", mismatch);
	     $getpat_close(filein);
	     $finish;
	  end

	$display ("Read line %0d", r);

 	// Test the output every clock edge to catch all transistions
	// Use the opposite edge from the one used to read the data
	// so that the outputs have stabilized
	@(posedge clock)
	if (out_data !== out_data_expected)
	  begin
             $display ("Mismatch on out_data: Expected: %h   Actual: %h\n",
		       out_data_expected, out_data);
	     mismatch = mismatch + 1;
	  end
	else
	  begin
             $display ("Test passed on out_data.  Expected: %h   Actual: %h\n",
		       out_data_expected, out_data);
	  end
     end

endmodule


module COUNT (clock,rst, in_flag, in_data, out_data);

   input clock, rst, in_flag;
   input [7:0] in_data;
   output [7:0] out_data;
   reg [7:0] 	out_data;
   
   always @(posedge clock or negedge rst)
     begin
	if (!rst)
	  out_data = 0;
	else
	  begin
	     if (in_flag) out_data = in_data;
	     else         out_data = out_data + 1;
	  end
     end

endmodule // COUNT

